# Debugging
This document describes how to debug the engine.  The engine is a binary
executable written in C, and the UI is written in Python.  At the time of
this writing, they communicate using LibLo, and OSC library over sockets.
In the future, this may be replaced with a solution that uses sockets
directly.

Note that the Mac and Windows builds use the engine compiled as a shared
library, the options for debugging on those platforms is limited.

# Logs
Adding logs is a good way to debug.  Log files are kept in
`$HOME/stargate/log/`.  Note that you cannot add logs to anything in the main
loop of the engine, as this will degrade performance.
## Logging in C
```c
// Becomes an ERROR level log line
fprintf(stderr, "Some message with an int: %i\n", 123);

// Becomes an INFO level log line
printf("Some message with another string: %s\n", "something");
```

## Logging in Python
```python
from sglib.log import LOG

LOG.info("something")
LOG.warning("something else")
LOG.error("Need to fix this")
try:
    raise Exception
except Exception as ex:
    LOG.exception(ex)
```

# Unit tests
This is the most preferred way to debug.  Write a test, check the coverage
report generated by `make test`.  Of course, if you do not actually know
what is wrong, you may need to discover what it is by replicating the bug
in GDB or Valgrind.

# Valgrind, GDB and other tools
To use these tools, one can compile a binary with the required flags, kill
the existing engine, and run the desired binary through the desired tools.
For example:
```
cd [stargate repo folder]/src
# Start Stargate, wait for it to load
scripts/stargate
# In a separate terminal or tmux window
pkill stargate-engine
# Note that the version of the engine with the -dbg suffix is preferred
# for use with GDB or LLDB
gdb ./stargate-engine-dbg
# Set some breakpoints
b some_function
b file:123 if some_var > 1234
# In Stargate, go to the main transport menu, select
#   Developer > Copy to clipboard > GDB run command
# To get a suitable command to paste into this terminal
run ...

# Or run Valgrind
# --tool=memcheck (the default) might be unusably slow, but the other
# tools such as callgrind and cachegrind can be useful.
# There is another action to copy the Valgrind command in the Developer
# section of the main menu
valgrind ./engine/stargate-engine-dbg ...
```
Note that the tools may slow down or block the process, which may cause
problems with audio hardware if `--no-hardware` or `--sleep`(implies
`--no-hardware`)  are not used.

